//=============================================================================
//
// file :        Standa8SMC3.h
//
// description : Include for the Standa8SMC3 class.
//
// project :	Standa 8SMC3-RS232 Stepper Motor Controller
//
// $Author: graziano $
//
// $Revision: 1.5 $
//
// $Log: Standa8SMC3.h,v $
// Revision 1.5  2017-03-10 13:19:04  graziano
// tango 9
//
// Revision 1.3  2015-09-08 10:27:00  graziano
// added configurable switch polarity
//
// Revision 1.2  2014-10-03 09:33:54  graziano
// fixed homing polarity, direction, removed % 360
//
// Revision 1.1  2014-09-22 13:21:58  graziano
// first commit
//
// Revision 1.7  2013-08-23 08:58:58  graziano
// Device_4Impl
//
// Revision 1.6  2011-01-19 16:18:20  graziano
// handled rw db
//
// Revision 1.5  2009/09/23 15:11:26  graziano
// added attribute Position_home
//
// Revision 1.4  2009/09/23 14:46:29  graziano
// bug fixing
//
// Revision 1.3  2009/04/15 10:14:21  graziano
// fixed sleep and homing
//
// Revision 1.2  2009/04/02 13:46:18  graziano
// added command find_edge
//
// Revision 1.1  2008/11/19 10:39:26  graziano
// first version
//
//
// copyleft :    European Synchrotron Radiation Facility
//               BP 220, Grenoble 38043
//               FRANCE
//
//=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================
#ifndef _STANDA8SMC3_H
#define _STANDA8SMC3_H

#include <tango.h>
#include <sys/time.h>
#include <math.h>	//for modf
//using namespace Tango;

/**
 * @author	$Author: graziano $
 * @version	$Revision: 1.5 $
 */

 //	Add your own constants definitions here.
 //-----------------------------------------------
 //defines from Serial.h
#define SL_RAW        0     /* raw read/write mode */
#define SL_NCHAR      1     /* character read/write mode */
#define SL_LINE       2     /* line read mode */
#define SL_RETRY      3     /* retry read mode */

#define PREFIX					"A"

#define READ_MIN_VELOCITY		"tl"		//tell min velocity
#define READ_MAX_VELOCITY		"tv"		//tell max velocity
#define READ_ACCELERATION		"ta"		//tell acceleration
#define READ_MICRO_STEP_CONF	"tr"		//tell microstep regime
#define READ_STEPS				"tp"		//tell counted steps after motor start command at the moment
#define READ_STEPS_SX			"td"		//tell steps that was written with AXsx command
#define READ_STEPS_ST			"ti"		//tell steps that was written with AXst command
#define READ_STATUS				"ts"		//tell motor status
#define READ_LIMIT_1			"t1"		//tell limit swtich 1 status
#define READ_LIMIT_2			"t2"		//tell limit swtich 2 status

#define CMD_MOVE				"go"		//start motor
#define CMD_OFF					"mf"		//motor off
#define CMD_ON					"mn"		//motor on
#define CMD_STANDBY				"mb"		//motor standby
#define CMD_STOP				"ms"		//motor stop
#define CMD_RESET				"mr"		//reset to 0=left,1=right limit switch

#define SET_MIN_VELOCITY		"sl"		//set start velocity, followed by [0-F][0-F] (FF=min, 01=max)
#define SET_MAX_VELOCITY		"sv"		//set max velocity, followed by [0-F][0-F] (FF=min, 01=max)
#define SET_ACCELERATION		"sa"		//set motor acceleration, followed by [0-F][0-F] (FF=min, 01=max)
#define SET_MICRO_STEP_CONF		"sr"		//set micro steps regime, followed by [1-4] (1=full step, 2=half step, 3=1/4 step, 4=1/8 step)
#define SET_LIMIT_ON			"sn"		//limit switch on, followed by [1-2] (1=limit switch 1, 2=limit switch 2)
#define SET_LIMIT_OFF			"sf"		//limit switch off, followed by [1-2] (1=limit switch 1, 2=limit switch 2)
#define SET_ROTATION_WAY		"sh"		//set rotation way, followed by [0-1] (0=to left, 1=to right)
#define SET_STEP_NUMBER			"sx"		//set steps number, followed by [0-F][0-F][0-F][0-F][0-F][0-F] (es. A0sx001A00 = set 6656 steps)
#define SET_OFFSET_STEP			"st"		//set number offset steps from limit switches, followed by [0-F][0-F][0-F][0-F] (es. A0sx001A00 = set 6656 steps)
#define SET_LS_POLARITY		"sb"		//set limit switch polarity to Low (0) or High (1)

#define ENABLE_SWITCH			"sn"		//reset to 1=left, 2=right limit switch

//#define POSITION_INVALID		-1000000000

static long speeds_table[256] = {
	-1,	//not allowed
	9091,8000,7407,7143,6667,6250,5797,5714,5195,5000,4762,4545,4348,4167,4000,3846,3704,
	3571,3448,3333,3226,3125,3030,2941,2857,2778,2703,2632,2564,2500,2439,2381,2326,2273,
	2222,2174,2128,2083,2041,2000,1961,1923,1887,1852,1818,1786,1754,1724,1695,1667,1639,
	1613,1587,1563,1538,1515,1493,1471,1449,1429,1408,1389,1370,1351,1333,1316,1299,1282,
	1266,1250,1235,1220,1205,1190,1176,1163,1149,1136,1124,1111,1099,1087,1075,1064,1053,
	1042,1031,1020,1010,1000,990,980,971,962,952,943,935,926,917,909,901,893,
	885,877,870,862,855,847,840,833,826,820,813,806,800,794,787,781,775,
	769,763,758,752,746,741,735,730,725,719,714,709,704,699,694,690,685,
	680,676,671,667,662,658,654,649,645,641,637,633,629,625,621,617,613,
	610,606,602,599,595,592,588,585,581,578,575,571,568,565,562,559,556,
	552,549,546,543,541,538,535,532,529,526,524,521,518,515,513,510,508,
	505,503,500,498,495,493,490,488,485,483,481,478,476,474,472,469,467,
	465,463,461,459,457,455,452,450,448,446,444,442,441,439,437,435,433,
	431,429,427,426,424,422,420,418,417,415,413,412,410,408,407,405,403,
	402,400,398,397,395,394,392,391,389,388,386,385,383,382,380,379,377
};

class standathread;

namespace Standa8SMC3_ns
{

/**
 * Class Description:
 * Device Server for the Standa 8SMC3-RS232 Stepper Motor Controller
 */

/*
 *	Device States Description:
*  Tango::OFF :
*  Tango::MOVING :
*  Tango::STANDBY :
*  Tango::FAULT :
 */


class Standa8SMC3: public TANGO_BASE_CLASS
{
public :
	//	Add your own data members here
	//-----------------------------------------
	friend class standathread;

	//	Here is the Start of the automatic code generation part
	//-------------------------------------------------------------	
/**
 *	@name attributes
 *	Attributs member data.
 */
//@{
		Tango::DevLong	*attr_Stat_read;
		Tango::DevDouble	*attr_Position_read;
		Tango::DevDouble	attr_Position_write;
		Tango::DevDouble	*attr_Acceleration_read;
		Tango::DevDouble	attr_Acceleration_write;
		Tango::DevDouble	*attr_Accuracy_read;
		Tango::DevDouble	attr_Accuracy_write;
		Tango::DevDouble	*attr_Position_home_read;
		Tango::DevDouble	attr_Position_home_write;
		Tango::DevDouble	*attr_Speed_read;
		const Tango::DevDouble	*attr_Speed_write;
//@}

/**
 *	@name Device properties
 *	Device properties member data.
 */
//@{
/**
 *	Number of the Axis on the motion controller
 */
	Tango::DevLong	axisNumber;
/**
 *	This is the Motion controller software forward limit
 *	Stops any movement passing that limit
 */
	Tango::DevLong	axisForwardLimit;
/**
 *	Motion controller software backward limit.
 */
	Tango::DevLong	axisBackwardLimit;
/**
 *	the serial device used for communications with Standa Stepper Motor controller
 */
	string	serialdevice;
/**
 *	Serial port timeout in ms
 */
	Tango::DevLong	serial_timeout;
/**
 *	Sleep time in ms between serial write of a command and serial read of the response
 */
	Tango::DevLong	wr_sleep;
/**
 *	Sleep time in ms between threads
 */
	Tango::DevLong	th_sleep;
/**
 *	the ratio between hardware units and user units
 */
	Tango::DevDouble	positionRatio;
/**
 *	Max speed (steps/s): 377 min value, 9091 max value
 */
	Tango::DevLong	defaultMaxSpeed;
/**
 *	Default Min speed: 377 min value, 9091 max value
 */
	Tango::DevLong	defaultMinSpeed;
/**
 *	Default acceleration: 255 min value, 01 max value
 */
	Tango::DevLong	defaultAcceleration;
/**
 *	Max allowed speed (steps/s): 9091 highest value, 377 lowest value
 */
	Tango::DevLong	maxAllowedSpeed;
/**
 *	current position of the rotation stage
 */
	Tango::DevDouble	current_position;
/**
 *	Limit Switch Polarity Active Low (0) or Active High (1)
 */
	Tango::DevLong	limit_switch_polarity;
/**
 *	Memorized position of the rotation stage
 */
	Tango::DevDouble	memo_position;
//@}

/**@name Constructors
 * Miscellaneous constructors */
//@{
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	Standa8SMC3(Tango::DeviceClass *cl,string &s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device Name
 */
	Standa8SMC3(Tango::DeviceClass *cl,const char *s);
/**
 * Constructs a newly allocated Command object.
 *
 *	@param cl	Class.
 *	@param s 	Device name
 *	@param d	Device description.
 */
	Standa8SMC3(Tango::DeviceClass *cl,const char *s,const char *d);
//@}

/**@name Destructor
 * Only one desctructor is defined for this class */
//@{
/**
 * The object desctructor.
 */	
	~Standa8SMC3() {delete_device();};
/**
 *	will be called at device destruction or at init command.
 */
	void delete_device();
//@}

	
/**@name Miscellaneous methods */
//@{
/**
 *	Initialize the device
 */
	virtual void init_device();
/**
 *	Always executed method befor execution command method.
 */
	virtual void always_executed_hook();

//@}

/**
 * @name Standa8SMC3 methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for Stat acquisition result.
 */
	virtual void read_Stat(Tango::Attribute &attr);
/**
 *	Extract real attribute values for Position acquisition result.
 */
	virtual void read_Position(Tango::Attribute &attr);
/**
 *	Write Position attribute values to hardware.
 */
	virtual void write_Position(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Acceleration acquisition result.
 */
	virtual void read_Acceleration(Tango::Attribute &attr);
/**
 *	Write Acceleration attribute values to hardware.
 */
	virtual void write_Acceleration(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Accuracy acquisition result.
 */
	virtual void read_Accuracy(Tango::Attribute &attr);
/**
 *	Write Accuracy attribute values to hardware.
 */
	virtual void write_Accuracy(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Position_home acquisition result.
 */
	virtual void read_Position_home(Tango::Attribute &attr);
/**
 *	Write Position_home attribute values to hardware.
 */
	virtual void write_Position_home(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for Speed acquisition result.
 */
	virtual void read_Speed(Tango::Attribute &attr);
/**
 *	Write Speed attribute values to hardware.
 */
	virtual void write_Speed(Tango::WAttribute &attr);
/**
 *	Read/Write allowed for Stat attribute.
 */
	virtual bool is_Stat_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Position attribute.
 */
	virtual bool is_Position_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Acceleration attribute.
 */
	virtual bool is_Acceleration_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Accuracy attribute.
 */
	virtual bool is_Accuracy_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Position_home attribute.
 */
	virtual bool is_Position_home_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for Speed attribute.
 */
	virtual bool is_Speed_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for Forward command.
 */
	virtual bool is_Forward_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Backward command.
 */
	virtual bool is_Backward_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for Stop command.
 */
	virtual bool is_Stop_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for MotorOFF command.
 */
	virtual bool is_MotorOFF_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for MotorON command.
 */
	virtual bool is_MotorON_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for MotorSTANDBY command.
 */
	virtual bool is_MotorSTANDBY_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GoHome command.
 */
	virtual bool is_GoHome_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for SetHome command.
 */
	virtual bool is_SetHome_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GoToPosition command.
 */
	virtual bool is_GoToPosition_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for FindEdge command.
 */
	virtual bool is_FindEdge_allowed(const CORBA::Any &any);
/**
 * move forward for argin offset
 *	@param	argin	relative movement in user units
 *	@exception DevFailed
 */
	void	forward(Tango::DevDouble);
/**
 * move backward for argin offset
 *	@param	argin	relative movement in user units
 *	@exception DevFailed
 */
	void	backward(Tango::DevDouble);
/**
 * 
 *	@exception DevFailed
 */
	void	stop();
/**
 * disable the power amplifier of the motor
 *	@exception DevFailed
 */
	void	motor_off();
/**
 * enables the power amplifier of the motor
 *	@exception DevFailed
 */
	void	motor_on();
/**
 * 
 *	@exception DevFailed
 */
	void	motor_standby();
/**
 * move to home position (if defined)
 *	@exception DevFailed
 */
	void	go_home();
/**
 * Set current position as home
 *	@exception DevFailed
 */
	void	set_home();
/**
 * go to absolute position (if home defined)
 *	@param	argin	absolute position in user units
 *	@exception DevFailed
 */
	void	go_to_position(Tango::DevDouble);
/**
 * Move to left (argin=0) or right (argin=1) limit switch
 *	@param	argin	0=find left limit switch 1=find right limit switch
 *	@exception DevFailed
 */
	void	find_edge(Tango::DevLong);

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

	//	Here is the end of the automatic code generation part
	//-------------------------------------------------------------	


private:
	string host_rw;
	long port_rw;
protected :	
	//	Add your own data members here
	//-----------------------------------------
	int timeoutHiPrio;
	int timeoutLowPrio;	

	standathread *readloop;
	
	void write_serial(char *, long);
//	void write_serial(string);
	void read_serial(char */*, long*/);	
	
public:
	Tango::DeviceProxy *devserial;	
	int abortflag;  // flag used to abort reading loop
	int stat;
	double position_tmp, position_moving;
	bool position_valid;
	bool moving;
	//long old_homing_pos;
	bool homing;
	int step_factor;
	int direction;		//1 = forward, -1 = backward
	Tango::DevState old_state;
	string old_status;
	omni_mutex::omni_mutex *mutex;
	char positive_rotation_way[4];
	char negative_rotation_way[4];
	Tango::DevLong homing_direction;
	
	int WriteReadStanda(string cmd, string value, string &argout, bool has_resp);
	
	int check_timeout(timeval, int);	
	void abort_sleep(double time);	
};

}	// namespace_ns

#endif	// _STANDA8SMC3_H
