//
// file :         standathread.cpp
//
// description :  reading thread
//
// project :      TANGO Device Server
//
// $Author: graziano $ 
//
// $Revision: 1.4 $
//
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//

#include "standathread.h"
static const char __FILE__rev[] = __FILE__ " $Revision: 1.4 $";

//+------------------------------------------------------------------
//
//	method:			standathread::standathread()
//
//	description:	standathread constructor
//
//-------------------------------------------------------------------
standathread::standathread(Standa8SMC3_ns::Standa8SMC3 *s):Tango::LogAdapter(s)
{ 
	DEBUG_STREAM << "standathread::standathread(): constructor... !" << endl;
	mystanda = s;
	mutex_sleep = new omni_mutex::omni_mutex();
	
}

//+------------------------------------------------------------------
//
//	method:			standathread::~standathread()
//
//	description:	standathread destructor
//
//-------------------------------------------------------------------
standathread::~standathread()
{
	DEBUG_STREAM << "standathread::~standathread(): destructor... !" << endl;

	//mystanda->loopterminatedflag = 1;
	delete mutex_sleep;
	DEBUG_STREAM << "standathread::~standathread(): deleted... !" << endl;
}

//+------------------------------------------------------------------
//
//	method:			standathread::run()
//
//	description:	Run
//
//-------------------------------------------------------------------
void standathread::run(void *) 
{
	INFO_STREAM << "standathread::run(): running... !" << endl;	
	int pausesec,pausenano,
	    cnt_err = 0,
		max_cnt_err = 3,
		ret;
//	im540_map::iterator it;// = myim540->read_data.begin();
	
	string id;
	//update_scheduler();
	string resp;
	int axis;
	int stat;
	int steps;
	int count = 0;
	Tango::DevState stat_moving;
	mutex_sleep->lock();
	pausesec = pausesec_dfl = (int)((5.0 * (double)mystanda->th_sleep)/1000);					//poll status slow when not moving
	pausenano = pausenano_dfl = ((5 * mystanda->th_sleep)-pausesec_dfl*1000)*1000000;
	mutex_sleep->unlock();
	
	while (!mystanda->abortflag) 
	{	
		ret = 0;
		count ++;
		if(mystanda->WriteReadStanda(READ_STATUS, "",resp, true))
		{
			WARN_STREAM << "standathread::run(): exception reading steps status: '" << resp << "'" << endl;
			ret = -1;
		}		
		if(resp.length() >= 12)
		{
			sscanf(resp.c_str(), "%i:0%i{%x}",&axis,&stat,&steps);
			DEBUG_STREAM << "standathread::run(): read status: stat=" << stat << " axis=" << axis << " steps=" << steps << endl;
			mystanda->stat = stat;
			//mystanda->position_tmp = steps;
			if(stat == 1 && !mystanda->homing)
			{
				if(mystanda->get_state()!=Tango::MOVING)
				{
					mystanda->set_state(Tango::MOVING);	
					mystanda->set_status("Motor MOVING");
					/*vector<string> filterable_names;
					vector<double> filterable_values;*/
					stat_moving = Tango::MOVING;
					//mystanda->push_event("State", filterable_names, filterable_values, &stat_moving);
				}					
			}
			else if((mystanda->get_state() == Tango::MOVING || mystanda->moving) && !mystanda->homing)		//moving needed if movement too fast so status has not changed to Tango::MOVING
			{
				mutex_sleep->lock();
				pausesec_dfl = (int)((5 * (double)mystanda->th_sleep)/1000);					//poll status slow when not moving
				pausenano_dfl = ((5 * mystanda->th_sleep)-pausesec_dfl*1000)*1000000;
				mystanda->moving = false;
				mutex_sleep->unlock();				
				DEBUG_STREAM << "standathread::run(): finished moving: pausesec_dfl=" << pausesec_dfl << " pausenano_dfl=" << pausenano_dfl << endl;
				abort_sleep((double(3.0*mystanda->wr_sleep))/1000);
				//last read of position for this movement
				if(mystanda->WriteReadStanda(READ_STEPS,"",resp, true))
				{
					WARN_STREAM << "standathread::run(): exception reading total steps: '" << resp << "'" << endl;
					ret = -1;
				}
				
				if(resp.length() >= 3)
				{
					size_t pos = resp.find(":");
					string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
					long val = strtol(temp.c_str(),0,16);
					DEBUG_STREAM << "standathread::run(): reading total steps = " << temp << " val=" << val << endl;
					if(mystanda->position_valid)
					{
						mystanda->position_moving = mystanda->direction * ((double)(val) / (double)(mystanda->step_factor));
#if _MODULE_360
						if(mystanda->position_moving > (360 * mystanda->positionRatio))
							mystanda->position_moving -= (360 * mystanda->positionRatio);
						else if(mystanda->position_moving < (-360 * mystanda->positionRatio))
							mystanda->position_moving += (360 * mystanda->positionRatio);		
#endif
						DEBUG_STREAM << "standathread::run(): changing position_moving to = " << mystanda->position_moving << endl;
						mystanda->position_tmp += mystanda->position_moving;
						mystanda->position_moving = 0;
#if _MODULE_360
						do
						{
							if(mystanda->position_tmp >= (360 * mystanda->positionRatio))
								mystanda->position_tmp -= (360 * mystanda->positionRatio);
							else if(mystanda->position_tmp < (0 * mystanda->positionRatio))
								mystanda->position_tmp += (360 * mystanda->positionRatio);
						} while((mystanda->position_tmp >= (360 * mystanda->positionRatio)) || (mystanda->position_tmp < (0 * mystanda->positionRatio)));
#endif
						DEBUG_STREAM << "standathread::run(): changing position_tmp to = " << mystanda->position_tmp << endl;
					}					
				}
				
				mystanda->memo_position =  (mystanda->position_tmp + mystanda->position_moving) / (double)mystanda->positionRatio;
				
				mystanda->set_state(mystanda->old_state);
				mystanda->set_status(mystanda->old_status);
				try {
						mystanda->motor_off();
				} catch(...)
				{
					INFO_STREAM <<__func__<< ": exception in motor_off!()" << endl;
				}
			}
			else if((stat == 2 || stat == 3)  && mystanda->homing)
			{
				DEBUG_STREAM << "standathread::run(): continue homing..." << endl;
			}
			else if(stat != 2 && stat != 3 && mystanda->homing)
			{
				DEBUG_STREAM << "standathread::run(): finished homing!" << endl;
				//finally disable limit switches
				string resp;
				if(mystanda->WriteReadStanda(SET_LIMIT_OFF,"1",resp, true))
					WARN_STREAM << "standathread::run(): exception disabling left limit switch: '"<< resp << "'" << endl;	
				abort_sleep((double(mystanda->wr_sleep))/1000);
				if(mystanda->WriteReadStanda(SET_LIMIT_OFF,"2",resp, true))
					WARN_STREAM << "standathread::run(): exception disabling right limit switch: '"<< resp << "'" << endl;
				mystanda->homing = false;
				mystanda->set_state(mystanda->old_state);
				mystanda->set_status(mystanda->old_status);
				mystanda->set_home();
				try {
						mystanda->motor_off();
				} catch(...)
				{
					INFO_STREAM <<__func__<< ": exception in motor_off!()" << endl;
				}
			}
		}
		/*if(mystanda->get_state() == Tango::MOVING || mystanda->moving)
		{
			abort_sleep((double(mystanda->wr_sleep))/1000);
			if(mystanda->WriteReadStanda(READ_STEPS_SX,"",resp, true))
			{
				WARN_STREAM << "standathread::run(): exception reading steps sx: '" << resp << "'" << endl;
				ret = -1;
			}
			
			if(resp.length() >= 3)
			{
				size_t pos = resp.find(":");
				string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
				long val = strtol(temp.c_str(),0,16);
				DEBUG_STREAM << "standathread::run(): reading steps sx = "<< resp << " temp=" << temp << " val=" << val << endl;

			}		
		}*/
		if(mystanda->get_state() == Tango::MOVING || mystanda->moving || mystanda->homing)
		{
			abort_sleep((double(mystanda->wr_sleep))/1000);
			if(mystanda->WriteReadStanda(READ_STEPS,"",resp, true))
			{
				WARN_STREAM << "standathread::run(): exception reading total steps: '" << resp << "'" << endl;
				ret = -1;
			}
			
			if(resp.length() >= 3)
			{
				size_t pos = resp.find(":");
				string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
				long val = strtol(temp.c_str(),0,16);
				DEBUG_STREAM << "standathread::run(): reading total steps = " << temp << " val=" << val << endl;
				if(mystanda->position_valid)
				{
					mystanda->position_moving = mystanda->direction * ((double)(val) / (double)(mystanda->step_factor));
#if _MODULE_360
					if(mystanda->position_moving > (360 * mystanda->positionRatio))
						mystanda->position_moving -= (360 * mystanda->positionRatio);
					else if(mystanda->position_moving < (0 * mystanda->positionRatio))
						mystanda->position_moving += (360 * mystanda->positionRatio);
#endif
					/*do
					{
						if(mystanda->position_moving >= (360 * mystanda->positionRatio))
							mystanda->position_moving -= (360 * mystanda->positionRatio);
						else if(mystanda->position_moving < (0 * mystanda->positionRatio))
							mystanda->position_moving += (360 * mystanda->positionRatio);
					} while((mystanda->position_moving >= (360 * mystanda->positionRatio)) || (mystanda->position_moving < (0 * mystanda->positionRatio)));*/	
					DEBUG_STREAM << "standathread::run(): changing position_moving to = " << mystanda->position_moving << endl;
				}
			}		
		}
		else
		{
/*			if(count % 2 == 1)
			{
				abort_sleep((double(mystanda->wr_sleep))/1000);
				if(mystanda->WriteReadStanda(READ_STEPS,"",resp, true))
				{
					WARN_STREAM << "standathread::run(): exception reading total steps: '" << resp << "'" << endl;
					ret = -1;
				}
				
				if(resp.length() >= 3)
				{
					size_t pos = resp.find(":");
					string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
					long val = strtol(temp.c_str(),0,16);
					DEBUG_STREAM << "standathread::run(): reading total steps = " << temp << " val=" << val << endl;
					if(mystanda->position_valid)
					{
						mystanda->position_moving = mystanda->direction * ((double)(val) / (double)(mystanda->step_factor));
						if(mystanda->position_moving > (360 * mystanda->positionRatio))
							mystanda->position_moving -= (360 * mystanda->positionRatio);
						else if(mystanda->position_moving < (-360 * mystanda->positionRatio))
							mystanda->position_moving += (360 * mystanda->positionRatio);		
						DEBUG_STREAM << "standathread::run(): changing position_moving to = " << mystanda->position_moving << endl;
						mystanda->position_tmp += mystanda->position_moving;
						mystanda->position_moving = 0;
					}					
				}
			}
			else*/ if(count % 10 == 1)
			{
				abort_sleep((double(mystanda->wr_sleep))/1000);
				if(mystanda->WriteReadStanda(READ_ACCELERATION,"",resp, true))
				{
					WARN_STREAM << "standathread::run(): exception reading acceleration: '" << resp << "'" << endl;
					ret = -1;
					*mystanda->attr_Acceleration_read = -1;
				}
				
				if(resp.length() >= 5)
				{
					//size_t pos = resp.find(":12");//TODO: if 8smc1
					size_t pos = resp.find(":61");//TODO: if 8smc3
					string temp = resp.substr(pos +3,resp.length()-pos-5);		//eliminate ':12' and /d/a
					long val = strtol(temp.c_str(),0,16);
					*mystanda->attr_Acceleration_read = val;
					//DEBUG_STREAM << "standathread::run(): read = "<< resp << " temp=" << temp << " val=" << val << endl;			
				}
				else
					*mystanda->attr_Acceleration_read = -1;
			}
			else if(count % 10 == 4)
			{
				abort_sleep((double(mystanda->wr_sleep))/1000);
				if(mystanda->WriteReadStanda(READ_MIN_VELOCITY,"",resp, true))
				{
					WARN_STREAM << "standathread::run(): exception reading min speed: '" << resp << "'" << endl;
					mystanda->attr_Speed_read[0] = -1;
					ret = -1;
				}
				
				if(resp.length() >= 3)
				{
					size_t pos = resp.find(":");
					string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
					long val = strtol(temp.c_str(),0,16);
					if(val > 0 && val < 256)
					{
						//mystanda->attr_Speed_read[0] = ((double)speeds_table[val]) / (mystanda->positionRatio * (double)mystanda->step_factor);
						mystanda->attr_Speed_read[0] = speeds_table[val];
					}
					else
						mystanda->attr_Speed_read[0] = -1;
					//DEBUG_STREAM << "Standa8SMC3::read_Speed(): MIN read = "<< resp << " temp=" << temp << " val=" << val << endl;			
				}
				else
					mystanda->attr_Speed_read[0] = -1;
					
				abort_sleep((double(mystanda->wr_sleep))/1000);
				if(mystanda->WriteReadStanda(READ_MAX_VELOCITY,"",resp, true))
				{
					WARN_STREAM << "standathread::run(): exception reading max speed: '" << resp << "'" << endl;
					ret = -1;
					mystanda->attr_Speed_read[1] = -1;
				}
				
				if(resp.length() >= 3)
				{
					size_t pos = resp.find(":");
					string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
					long val = strtol(temp.c_str(),0,16);
					if(val > 0 && val < 256)
					{
						//mystanda->attr_Speed_read[1] = ((double)speeds_table[val]) / (mystanda->positionRatio * (double)mystanda->step_factor);
						mystanda->attr_Speed_read[1] = speeds_table[val];
					}
					else
						mystanda->attr_Speed_read[1] = -1;
					//DEBUG_STREAM << "Standa8SMC3::read_Speed(): MIN read = "<< resp << " temp=" << temp << " val=" << val << endl;			
				}
				else
					mystanda->attr_Speed_read[1] = -1;
			}
			else if(count % 10 == 8)
			{
				abort_sleep((double(mystanda->wr_sleep))/1000);
				if(mystanda->WriteReadStanda(READ_MICRO_STEP_CONF,"",resp, true))
				{
					WARN_STREAM << "standathread::run(): exception reading micro step config: '" << resp << "'" << endl;
					ret = -1;
					*mystanda->attr_Accuracy_read = -1;
				}
				
				if(resp.length() >= 4)
				{
					size_t pos = resp.find(":0");
					string temp = resp.substr(pos +2,resp.length()-pos-4);		//eliminate ':0' and /d/a
					long val = strtol(temp.c_str(),0,16);
					if(val==1)
						mystanda->step_factor = 1;
					else if(val==2)
						mystanda->step_factor = 2;
					else if(val==3)
						mystanda->step_factor = 4;
					else if(val==4)
						mystanda->step_factor = 8;			
					*mystanda->attr_Accuracy_read = val;
					//DEBUG_STREAM << "standathread::run(): read = "<< resp << " temp=" << temp << " val=" << val << endl;			
				}
				else
					*mystanda->attr_Accuracy_read = -1;
			}
			if(count % 60 == 30)
			{
				abort_sleep((double(mystanda->wr_sleep))/1000);
				Tango::DeviceData Din;
				Tango::DevLong flush = 2;  //flush input and output	
				Din << flush;
				try	{
					mystanda->devserial->command_inout("DevSerFlush",Din);
				}
				catch(Tango::DevFailed e) {
					string temp;
					temp = e.errors[0].desc;
					WARN_STREAM << "standathread::run(): error flushing serial device: " << temp << endl;
				}				
			}
		}
		
		if (ret < 0) 
			cnt_err++;
		else
			cnt_err = 0;	
		if (cnt_err > max_cnt_err) {
			pausesec = 5;
			pausenano = 0;
		}
		else {
			pausesec = pausesec_dfl;
			pausenano = pausenano_dfl;
		}
		//DEBUG_STREAM << "standathread::run(): going to sleep: pausesec_dfl=" << pausesec_dfl << " pausenano_dfl=" << pausenano_dfl << endl;		
		//omni_thread::sleep(pausesec,pausenano);
		abort_sleep(pausesec + ((double)pausenano)/1000000000);
		//abort_sleep(myim540->th_sleep/1000);		
	}
	INFO_STREAM << "standathread::run(): exit!!!" << endl;
}

//+------------------------------------------------------------------
//
//	method:			standathread::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void standathread::abort_sleep(double time) 
{		
	for (int i = 0; i < (time/0.1); i++) {
		if (mystanda->abortflag)
			break;
		omni_thread::sleep(0,100000000);
	}
}


